function [varargout]=gauss2gumb(m,sigma,n,N)
%GAUSS2GUMB Maximum of n Gaussian r.v:s tends to Gumbel when n->infty
%   The routine GAUSS2GUMB demonstrates that the maximum of n 
%   normally-distributed random variables tend to a Gumbel-distributed
%   random variable when n tends to infinity.
%
%   GAUSS2GUMB(M,SIGMA,n,N) simulates N samples, each containg
%   n normal (Gaussian) random numbers with expectation M and standard
%   deviation SIGMA ("x=M+SIGMA*randn(n,N)"). The maximum of each of
%   the n samples will in figure 1 be Gumbel-plotted using 'wgumbplot'
%   of the WAFO toolbox. In figure 2, the empirical distribution
%   function of the same maxima will be plotted (blue, solid,
%   "staircase" line) together with, firstly,
%   the Gumbel distribution function obtained from the ML estimates
%   of the maxima (red, dashed line) and with, secondly, the Gumbel
%   distribution function obtained from the asymptotic parameters
%   depending on n (green, solid line).
%
%   [ABASYMPTOTIC,ABML]=GAUSSGUMB(M,SIGMA,n,N) does the same thing.
%   However, estimates of a and b (scale parameter and location parameter,
%   respectively, in a Gumbel distribution) are given in ABASYMPTOTIC
%   (extreme-value asymptotics) and ABML (maximum-likelihood).
%
% Written by Tord Rikte 2002-xx-xx.
% Revised by Tord Rikte 2003-03-19--20.
%
x=m+sigma*randn(n,N);
y=max(x);
figure(1);
hold off
clf
wgumbplot(y);
grid on
figure(2);
hold off
clf
[abAs,covm]=wgumbfit(y);
hold on
an=sigma/sqrt(2*log(n));
bn=m+sigma*(sqrt(2*log(n))-(log(log(n))+log(4*pi))/(sqrt(2*log(n))*2));
yv=linspace(min(y),max(y),1000);
plot(yv,exp(-exp(-(yv-bn)/an)),'g')
hold off
grid
abML=[an bn];
if nargout==0
    disp(['ML method: a=' num2str(abML(1),16) ', b=' num2str(abML(2),16)])
    disp(['Extreme-value asymptotics: a=' num2str(abAs(1),16) ', b=' num2str(abAs(2),16)])
else
    varargout{1}=abAs;
    varargout{2}=abML;
end