function r = wgpdrnd(k,s,m0,varargin);
%WGPDRND Random matrices from a Generalized Pareto Distribution
% 
% CALL:  R = wgpdrnd(k,s,m,sz);
%     
%        R = matrix of random numbers
%        k = shape parameter in the GPD  (see wgpdcdf)
%        s = scale parameter in the GPD    (default 1)
%        m = location parameter in the GPD (default 0)
%        sz = size(R)    (Default common size of k, s and m)
%             sz can be a comma separated list or a vector 
%             giving the size of R (see zeros for options).
%
% The random numbers are generated by the inverse method. 
%
% Example:
%   R1=wgpdrnd(2,1,0,1,100);  % GPD k=2
%   R2=wgpdrnd(1,1,0,1,100);  % GPD k=1  ==>  Uniform
%   R3=wgpdrnd(0,1,0,1,100);  % GPD k=0  ==>  Exponential
%   plot([R1 R2 R3],'.')
%
% See also: wgpdinv, wgpdcdf

% Tested on: Matlab 5.3
% History: 
% Revised by jr 22.12.1999
% revised ms 14.06.2000
% - updated header info
% - changed name to wgpdrnd (from gpdrnd)
% - allowed 2 arguments
% revised pab 23.10.2000
%   - added default s,m0
%  - added comnsize, nargchk
%  - added greater flexibility on the sizing of R

error(nargchk(2,inf,nargin))
if nargin<2|isempty(s), s=1;end
if nargin<3|isempty(m0), m0=0;end
if nargin<4,
  [errorcode k ,s,m0] = comnsize(k,s,m0);
else
  [errorcode k,s,m0] = comnsize(k,s,m0,zeros(varargin{:}));
end
if errorcode > 0
  error('k,s and m0 must be of common size or scalar.');
end
r = wgpdinv(rand(size(k)),k,s,m0);

